# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import platform

# project
from kiwi.bootloader.config.base import BootLoaderConfigBase
from kiwi.bootloader.template.isolinux import BootLoaderTemplateIsoLinux
from kiwi.utils.sync import DataSync
from kiwi.logger import log
from kiwi.path import Path
from kiwi.defaults import Defaults

from kiwi.exceptions import (
    KiwiTemplateError,
    KiwiBootLoaderIsoLinuxPlatformError
)


class BootLoaderConfigIsoLinux(BootLoaderConfigBase):
    """
    isolinux bootloader configuration.

    Attributes

    * :attr:`terminal`
        terminal mode set to graphics by default

    * :attr:`gfxmode`
        configured or default graphics mode

    * :attr:`timeout`
        configured or default boot timeout

    * :attr:`cmdline`
        kernel boot arguments

    * :attr:`cmdline_failsafe`
        kernel failsafe boot arguments

    * :attr:`hypervisor_domain`
        configured hypervisor domain name or none

    * :attr:`multiboot`
        multiboot requested true|false

    * :attr:`isolinux`
        Instance of config template: BootLoaderTemplateIsoLinux

    * :attr:`config`
        Configuration data from template substitution

    * :attr:`config_message`
        Configuration text message if graphics mode failed
    """
    def post_init(self, custom_args):
        """
        isolinux post initialization method

        Setup class attributes
        """
        self.custom_args = custom_args
        arch = platform.machine()
        if arch == 'x86_64':
            self.arch = arch
        elif arch == 'i686' or arch == 'i586':
            self.arch = 'ix86'
        else:
            raise KiwiBootLoaderIsoLinuxPlatformError(
                'host architecture %s not supported for isolinux setup' % arch
            )

        self.terminal = self.xml_state.build_type.get_bootloader_console()
        self.gfxmode = self.get_gfxmode('isolinux')
        # isolinux counts the timeout in units of 1/10 sec
        self.timeout = self.get_boot_timeout_seconds() * 10
        self.cmdline = self.get_boot_cmdline()
        self.cmdline_failsafe = ' '.join(
            [self.cmdline, Defaults.get_failsafe_kernel_options()]
        )
        self.failsafe_boot = self.failsafe_boot_entry_requested()
        self.hypervisor_domain = self.get_hypervisor_domain()

        self.multiboot = False
        if self.hypervisor_domain:
            if self.hypervisor_domain == 'dom0':
                self.multiboot = True
            elif self.hypervisor_domain == 'domU':
                self.multiboot = False

        self.isolinux = BootLoaderTemplateIsoLinux()
        self.config = None
        self.config_message = None

    def write(self):
        """
        Write isolinux.cfg and isolinux.msg file
        """
        log.info('Writing isolinux.cfg file')
        config_dir = self._get_iso_boot_path()
        config_file = config_dir + '/isolinux.cfg'
        if self.config:
            Path.create(config_dir)
            with open(config_file, 'w') as config:
                config.write(self.config)

        config_file_message = config_dir + '/isolinux.msg'
        if self.config_message:
            with open(config_file_message, 'w') as config:
                config.write(self.config_message)

    def setup_install_image_config(
        self, mbrid, hypervisor='xen.gz', kernel='linux', initrd='initrd'
    ):
        """
        Create isolinux.cfg in memory from a template suitable to boot
        from an ISO image in BIOS boot mode

        :param string mbrid: mbrid file name on boot device
        :param string hypervisor: hypervisor name
        :param string kernel: kernel name
        :param string initrd: initrd name
        """
        # mbrid parameter is not used, the information is placed as the
        # application id when creating the iso filesystem. Thus not part
        # of the configuration file
        log.info('Creating isolinux install config file from template')
        parameters = {
            'default_boot': self.get_install_image_boot_default('isolinux'),
            'kernel_file': kernel,
            'initrd_file': initrd,
            'boot_options': self.cmdline,
            'failsafe_boot_options': self.cmdline_failsafe,
            'gfxmode': self.gfxmode,
            'boot_timeout': self.timeout,
            'title': self.get_menu_entry_install_title()
        }
        if self.multiboot:
            log.info('--> Using multiboot install template')
            parameters['hypervisor'] = hypervisor
            template = self.isolinux.get_multiboot_install_template(
                self.failsafe_boot, self._have_theme(), self.terminal
            )
        else:
            log.info('--> Using install template')
            template = self.isolinux.get_install_template(
                self.failsafe_boot, self._have_theme(), self.terminal
            )
        try:
            self.config = template.substitute(parameters)
            template = self.isolinux.get_install_message_template()
            self.config_message = template.substitute(parameters)
        except Exception as e:
            raise KiwiTemplateError(
                '%s: %s' % (type(e).__name__, format(e))
            )

    def setup_live_image_config(
        self, mbrid, hypervisor='xen.gz', kernel='linux', initrd='initrd'
    ):
        """
        Create isolinux.cfg in memory from a template suitable to boot
        a live system from an ISO image in BIOS boot mode

        :param string mbrid: mbrid file name on boot device
        :param string hypervisor: hypervisor name
        :param string kernel: kernel name
        :param string initrd: initrd name
        """
        # mbrid parameter is not used, the information is placed as the
        # application id when creating the iso filesystem. Thus not part
        # of the configuration file
        log.info('Creating isolinux live ISO config file from template')
        parameters = {
            'default_boot': self.get_menu_entry_title(plain=True),
            'kernel_file': kernel,
            'initrd_file': initrd,
            'boot_options': self.cmdline,
            'failsafe_boot_options': self.cmdline_failsafe,
            'gfxmode': self.gfxmode,
            'boot_timeout': self.timeout,
            'title': self.get_menu_entry_title(plain=True)
        }
        if self.multiboot:
            log.info('--> Using multiboot standard ISO template')
            parameters['hypervisor'] = hypervisor
            template = self.isolinux.get_multiboot_template(
                self.failsafe_boot, self._have_theme(), self.terminal
            )
        else:
            log.info('--> Using standard ISO template')
            template = self.isolinux.get_template(
                self.failsafe_boot, self._have_theme(), self.terminal
            )
        try:
            self.config = template.substitute(parameters)
            template = self.isolinux.get_message_template()
            self.config_message = template.substitute(parameters)
        except Exception as e:
            raise KiwiTemplateError(
                '%s: %s' % (type(e).__name__, format(e))
            )

    def setup_install_boot_images(self, mbrid, lookup_path=None):
        """
        Provide isolinux boot metadata

        The mbrid parameter is not used, because only isolinux
        loader binary and possible theming files are copied

        :param string mbrid: unused
        :param string lookup_path: custom module lookup path
        """
        self._copy_loader_data_to_boot_directory(lookup_path)

    def setup_live_boot_images(self, mbrid, lookup_path=None):
        """
        Provide isolinux boot metadata

        Calls setup_install_boot_images because no different action required
        """
        self.setup_install_boot_images(mbrid, lookup_path)

    def _copy_loader_data_to_boot_directory(self, lookup_path):
        if not lookup_path:
            lookup_path = self.root_dir
        loader_data = lookup_path + '/image/loader/'
        Path.create(self._get_iso_boot_path())
        data = DataSync(
            loader_data, self._get_iso_boot_path()
        )
        data.sync_data(
            options=['-z', '-a']
        )

    def _get_iso_boot_path(self):
        return self.root_dir + '/boot/' + self.arch + '/loader'

    def _have_theme(self):
        if os.path.exists(self._get_iso_boot_path() + '/bootlogo'):
            return True
        return False



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os

# project
from kiwi.utils.compress import Compress
from kiwi.logger import log
from kiwi.command import Command
from kiwi.system.kernel import Kernel
from kiwi.boot.image.base import BootImageBase


class BootImageDracut(BootImageBase):
    """
    Implements creation of dracut boot(initrd) images.
    """
    def prepare(self):
        """
        For dracut no further preparation steps are required.
        dracut is called as chroot operation in the system tree.
        No extra caller environment will be created in this case
        """
        pass

    def create_initrd(self, mbrid=None):
        """
        Call dracut to create the initrd and XZ compress the result

        :param object mbrid: unused
        """
        if self.is_prepared():
            log.info('Creating generic dracut initrd archive')
            kernel_info = Kernel(self.boot_root_directory)
            kernel_details = kernel_info.get_kernel(raise_on_not_found=True)
            Command.run(
                [
                    'chroot', self.boot_root_directory,
                    'dracut', '--force',
                    '--no-hostonly',
                    '--no-hostonly-cmdline',
                    '--no-compress',
                    os.path.basename(self.initrd_file_name),
                    kernel_details.version
                ]
            )
            Command.run(
                [
                    'mv',
                    self.boot_root_directory + '/' + os.path.basename(
                        self.initrd_file_name
                    ),
                    self.initrd_file_name
                ]
            )
            log.info(
                '--> xz compressing archive'
            )
            compress = Compress(self.initrd_file_name)
            compress.xz()
            self.initrd_filename = compress.compressed_filename



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
"""
usage: kiwi system prepare -h | --help
       kiwi system prepare --description=<directory> --root=<directory>
           [--allow-existing-root]
           [--ignore-repos]
           [--set-repo=<source,type,alias,priority>]
           [--add-repo=<source,type,alias,priority>...]
           [--obs-repo-internal]
           [--add-package=<name>...]
           [--delete-package=<name>...]
           [--set-container-derived-from=<uri>]
           [--set-container-tag=<name>]
       kiwi system prepare help

commands:
    prepare
        prepare and install a new system for chroot access
    prepare help
        show manual page for prepare command

options:
    --add-package=<name>
        install the given package name
    --add-repo=<source,type,alias,priority>
        add repository with given source, type, alias and priority.
    --delete-package=<name>
        delete the given package name
    --allow-existing-root
        allow to use an existing root directory. Use with caution
        this could cause an inconsistent root tree if the existing
        contents does not fit to the additional installation
    --description=<directory>
        the description must be a directory containing a kiwi XML
        description and optional metadata files
    --ignore-repos
        ignore all repos from the XML configuration
    --obs-repo-internal
        when using obs:// repos resolve them using the SUSE internal
        buildservice. This only works if access to SUSE's internal
        buildservice is granted
    --root=<directory>
        the path to the new root directory of the system
    --set-container-derived-from=<uri>
        overwrite the source location of the base container
        for the selected image type. The setting is only effective
        if the configured image type is setup with an initial
        derived_from reference
    --set-container-tag=<name>
        overwrite the container tag in the container configuration.
        The setting is only effective if the container configuraiton
        provides an initial tag value
    --set-repo=<source,type,alias,priority>
        overwrite the repo source, type, alias or priority for the first
        repository in the XML description
"""
import os

# project
from kiwi.tasks.base import CliTask
from kiwi.help import Help
from kiwi.privileges import Privileges
from kiwi.system.prepare import SystemPrepare
from kiwi.system.setup import SystemSetup
from kiwi.defaults import Defaults
from kiwi.system.profile import Profile
from kiwi.logger import log


class SystemPrepareTask(CliTask):
    """
    Implements preparation and installation of a new root system

    Attributes

    * :attr:`manual`
        Instance of Help
    """
    def process(self):
        """
        Prepare and install a new system for chroot access
        """
        self.manual = Help()
        if self._help():
            return

        Privileges.check_for_root_permissions()

        self.load_xml_description(
            self.command_args['--description']
        )

        abs_root_path = os.path.abspath(self.command_args['--root'])

        self.runtime_checker.check_consistent_kernel_in_boot_and_system_image()
        self.runtime_checker.check_boot_image_reference_correctly_setup()
        self.runtime_checker.check_docker_tool_chain_installed()
        self.runtime_checker.check_volume_setup_has_no_root_definition()
        self.runtime_checker.check_image_include_repos_http_resolvable()
        self.runtime_checker.check_target_directory_not_in_shared_cache(
            abs_root_path
        )

        if self.command_args['--ignore-repos']:
            self.xml_state.delete_repository_sections()

        if self.command_args['--set-repo']:
            (repo_source, repo_type, repo_alias, repo_prio) = \
                self.quadruple_token(self.command_args['--set-repo'])
            self.xml_state.set_repository(
                repo_source, repo_type, repo_alias, repo_prio
            )

        if self.command_args['--add-repo']:
            for add_repo in self.command_args['--add-repo']:
                (repo_source, repo_type, repo_alias, repo_prio) = \
                    self.quadruple_token(add_repo)
                self.xml_state.add_repository(
                    repo_source, repo_type, repo_alias, repo_prio
                )

        if self.command_args['--set-container-tag']:
            self.xml_state.set_container_config_tag(
                self.command_args['--set-container-tag']
            )

        if self.command_args['--set-container-derived-from']:
            self.xml_state.set_derived_from_image_uri(
                self.command_args['--set-container-derived-from']
            )

        self.runtime_checker.check_repositories_configured()

        if Defaults.is_obs_worker():
            # This build runs inside of a buildservice worker. Therefore
            # the repo defintions is adapted accordingly
            self.xml_state.translate_obs_to_suse_repositories()

        elif self.command_args['--obs-repo-internal']:
            # This build should use the internal SUSE buildservice
            # Be aware that the buildhost has to provide access
            self.xml_state.translate_obs_to_ibs_repositories()

        package_requests = False
        if self.command_args['--add-package']:
            package_requests = True
        if self.command_args['--delete-package']:
            package_requests = True

        log.info('Preparing system')
        system = SystemPrepare(
            self.xml_state,
            abs_root_path,
            self.command_args['--allow-existing-root']
        )
        manager = system.setup_repositories()
        system.install_bootstrap(manager)
        system.install_system(
            manager
        )
        if package_requests:
            if self.command_args['--add-package']:
                system.install_packages(
                    manager, self.command_args['--add-package']
                )
            if self.command_args['--delete-package']:
                system.delete_packages(
                    manager, self.command_args['--delete-package']
                )

        profile = Profile(self.xml_state)

        defaults = Defaults()
        defaults.to_profile(profile)

        setup = SystemSetup(
            self.xml_state, abs_root_path
        )
        setup.import_shell_environment(profile)

        setup.import_description()
        setup.import_overlay_files()
        setup.import_image_identifier()
        setup.setup_groups()
        setup.setup_users()
        setup.setup_keyboard_map()
        setup.setup_locale()
        setup.setup_timezone()

        system.pinch_system(
            manager=manager, force=True
        )

        # make sure manager instance is cleaned up now
        del manager

        # setup permanent image repositories after cleanup
        if self.xml_state.has_repositories_marked_as_imageinclude():
            setup.import_repositories_marked_as_imageinclude()
        setup.call_config_script()

        # make sure system instance is cleaned up now
        del system

    def _help(self):
        if self.command_args['help']:
            self.manual.show('kiwi::system::prepare')
        else:
            return False
        return self.manual



from mock import patch

import mock

from .test_helper import raises

from kiwi.exceptions import (
    KiwiUriStyleUnknown,
    KiwiUriTypeUnknown
)

from kiwi.system.uri import Uri

import hashlib


class TestUri(object):
    @classmethod
    def setup_class(self):
        self.mock_mkdtemp = mock.Mock()
        self.mock_manager = mock.Mock()

    @raises(KiwiUriStyleUnknown)
    def test_is_remote_raises_style_error(self):
        uri = Uri('xxx', 'rpm-md')
        uri.is_remote()

    @raises(KiwiUriTypeUnknown)
    def test_is_remote_raises_type_error(self):
        uri = Uri('xtp://download.example.com', 'rpm-md')
        uri.is_remote()

    @raises(KiwiUriStyleUnknown)
    def test_translate_unknown_style(self):
        uri = Uri('xxx', 'rpm-md')
        uri.translate()

    @raises(KiwiUriStyleUnknown)
    def test_translate_unsupported_style(self):
        uri = Uri('ms://foo', 'rpm-md')
        uri.translate()

    def test_is_remote(self):
        uri = Uri('https://example.com', 'rpm-md')
        assert uri.is_remote() is True
        uri = Uri('dir:///path/to/repo', 'rpm-md')
        assert uri.is_remote() is False

    def test_alias(self):
        uri = Uri('https://example.com', 'rpm-md')
        assert uri.alias() == hashlib.md5(
            'https://example.com'.encode()).hexdigest(
        )

    def test_credentials_file_name(self):
        uri = Uri(
            'http://example.com/foo?credentials=my_credentials&x=2',
            'rpm-md'
        )
        assert uri.credentials_file_name() == 'my_credentials'

    def test_credentials_default_file_name(self):
        uri = Uri(
            'http://example.com/foo',
            'rpm-md'
        )
        assert uri.credentials_file_name() == 'kiwiRepoCredentials'

    def test_translate_http_path_with_credentials(self):
        uri = Uri(
            'http://example.com/foo?credentials=kiwiRepoCredentials',
            'rpm-md'
        )
        assert uri.translate() == 'http://example.com/foo'

    def test_translate_ibs_project(self):
        uri = Uri('ibs://Devel:PubCloud/SLE_12_GA', 'rpm-md')
        assert uri.translate() == \
            'http://download.suse.de/ibs/Devel:/PubCloud/SLE_12_GA'

    def test_translate_obs_distro(self):
        uri = Uri('obs://13.2/repo/oss', 'yast2')
        assert uri.translate() == \
            'http://download.opensuse.org/distribution/13.2/repo/oss'

    def test_translate_obs_factory_distro(self):
        uri = Uri('obs://openSUSE:Factory/standard', 'yast2')
        assert uri.translate() == \
            'http://download.opensuse.org/tumbleweed/repo/oss'

    def test_translate_obs_project(self):
        uri = Uri('obs://Virt:Appliances/SLE_12', 'rpm-md')
        assert uri.translate() == \
            'http://download.opensuse.org/repositories/Virt:Appliances/SLE_12'

    def test_translate_dir_path(self):
        uri = Uri('dir:///some/path', 'rpm-md')
        assert uri.translate() == '/some/path'

    def test_translate_http_path(self):
        uri = Uri('http://example.com/foo', 'rpm-md')
        assert uri.translate() == 'http://example.com/foo'

    def test_translate_https_path(self):
        uri = Uri('https://example.com/foo', 'rpm-md')
        assert uri.translate() == 'https://example.com/foo'

    def test_translate_ftp_path(self):
        uri = Uri('ftp://example.com/foo', 'rpm-md')
        assert uri.translate() == 'ftp://example.com/foo'

    @patch('kiwi.system.uri.MountManager')
    @patch('kiwi.system.uri.mkdtemp')
    def test_translate_iso_path(self, mock_mkdtemp, mock_manager):
        mock_mkdtemp.return_value = 'tmpdir'
        manager = mock.Mock()
        manager.mountpoint = mock_mkdtemp.return_value
        mock_manager.return_value = manager
        uri = Uri('iso:///image/CDs/openSUSE-13.2-DVD-x86_64.iso', 'yast2')
        result = uri.translate()
        mock_manager.assert_called_once_with(
            device='/image/CDs/openSUSE-13.2-DVD-x86_64.iso',
            mountpoint='tmpdir'
        )
        manager.mount.assert_called_once_with()
        assert result == 'tmpdir'
        uri.mount_stack = []

    def test_translate_suse_buildservice_path(self):
        uri = Uri('suse://openSUSE:13.2/standard', 'yast2')
        assert uri.translate() == \
            '/usr/src/packages/SOURCES/repos/openSUSE:13.2/standard'

    def test_translate_suse_buildservice_container_path(self):
        uri = Uri('suse://project/repo/container#latest', 'container')
        assert uri.translate() == \
            '/usr/src/packages/SOURCES/containers/project/repo/container#latest'

    def test_translate_buildservice_obsrepositories_container_path(self):
        uri = Uri('obsrepositories:/container#latest', 'container')
        assert uri.translate() == \
            '/usr/src/packages/SOURCES/containers/_obsrepositories/container#latest'

    @patch('kiwi.system.uri.MountManager')
    @patch('kiwi.system.uri.mkdtemp')
    @patch('kiwi.system.uri.Path.wipe')
    def test_destructor(self, mock_wipe, mock_mkdtemp, mock_manager):
        mock_mkdtemp.return_value = 'tmpdir'
        manager = mock.Mock()
        manager.mountpoint = mock_mkdtemp.return_value
        manager.is_mounted = mock.Mock(
            return_value=True
        )
        mock_manager.return_value = manager
        uri = Uri('iso:///image/CDs/openSUSE-13.2-DVD-x86_64.iso', 'yast2')
        uri.translate()
        uri.__del__()
        manager.umount.assert_called_once_with()
        mock_wipe.assert_called_once_with(manager.mountpoint)



from mock import patch

import mock

import kiwi

from .test_helper import raises

from collections import namedtuple

from kiwi.exceptions import KiwiPxeBootImageError
from kiwi.builder.pxe import PxeBuilder


class TestPxeBuilder(object):
    @patch('kiwi.builder.pxe.FileSystemBuilder')
    @patch('kiwi.builder.pxe.BootImage')
    def setup(self, mock_boot, mock_filesystem):
        self.setup = mock.Mock()
        kiwi.builder.pxe.SystemSetup = mock.Mock(
            return_value=self.setup
        )
        self.boot_image_task = mock.MagicMock()
        self.boot_image_task.boot_root_directory = 'initrd_dir'
        mock_boot.return_value = self.boot_image_task
        self.filesystem = mock.MagicMock()
        self.filesystem.filename = 'myimage.fs'
        mock_filesystem.return_value = self.filesystem
        self.xml_state = mock.Mock()
        self.xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )
        self.xml_state.xml_data.get_name = mock.Mock(
            return_value='some-image'
        )
        kernel_type = namedtuple(
            'kernel', ['filename', 'version']
        )
        xen_type = namedtuple(
            'xen', ['filename', 'name']
        )
        self.kernel = mock.Mock()
        self.kernel.get_kernel = mock.Mock(
            return_value=kernel_type(filename='some-kernel', version='42')
        )
        self.kernel.get_xen_hypervisor = mock.Mock(
            return_value=xen_type(filename='hypervisor', name='xen.gz')
        )
        kiwi.builder.pxe.Kernel = mock.Mock(
            return_value=self.kernel
        )
        self.pxe = PxeBuilder(
            self.xml_state, 'target_dir', 'root_dir'
        )
        self.machine = mock.Mock()
        self.machine.get_domain = mock.Mock(
            return_value='dom0'
        )
        self.pxe.machine = self.machine
        self.pxe.image_name = 'myimage'

    @patch('kiwi.builder.pxe.Checksum')
    @patch('kiwi.builder.pxe.Compress')
    @patch('kiwi.logger.log.warning')
    @patch('kiwi.builder.pxe.Command.run')
    @patch('os.rename')
    def test_create(
        self, mock_rename, mock_command, mock_log_warn,
        mock_compress, mock_checksum
    ):
        compress = mock.Mock()
        mock_compress.return_value = compress
        compress.compressed_filename = 'compressed-file-name'
        checksum = mock.Mock()
        mock_checksum.return_value = checksum
        self.boot_image_task.required = mock.Mock(
            return_value=True
        )
        self.pxe.create()
        self.filesystem.create.assert_called_once_with()
        mock_rename.assert_called_once_with(
            'myimage.fs', 'myimage'
        )
        compress.xz.assert_called_once_with()
        checksum.md5.assert_called_once_with('compressed-file-name.md5')
        self.boot_image_task.prepare.assert_called_once_with()
        self.setup.export_modprobe_setup.assert_called_once_with(
            'initrd_dir'
        )
        self.boot_image_task.create_initrd.assert_called_once_with()
        self.setup.export_rpm_package_list.assert_called_once_with(
            'target_dir'
        )
        self.setup.export_rpm_package_verification.assert_called_once_with(
            'target_dir'
        )
        # warning for not implemented pxedeploy handling
        assert mock_log_warn.called

    @patch('kiwi.builder.pxe.Checksum')
    @patch('kiwi.builder.pxe.Compress')
    @patch('os.rename')
    @raises(KiwiPxeBootImageError)
    def test_create_no_kernel_found(
        self, mock_rename, mock_compress, mock_checksum
    ):
        compress = mock.Mock()
        mock_compress.return_value = compress
        compress.compressed_filename = 'compressed-file-name'
        self.kernel.get_kernel.return_value = False
        self.pxe.create()

    @patch('kiwi.builder.pxe.Checksum')
    @patch('kiwi.builder.pxe.Compress')
    @patch('os.rename')
    @raises(KiwiPxeBootImageError)
    def test_create_no_hypervisor_found(
        self, mock_rename, mock_compress, mock_checksum
    ):
        compress = mock.Mock()
        mock_compress.return_value = compress
        compress.compressed_filename = 'compressed-file-name'
        self.kernel.get_xen_hypervisor.return_value = False
        self.pxe.create()



from mock import call
from mock import patch

from kiwi.container.docker import ContainerImageDocker


class TestContainerImageDocker(object):

    @patch('kiwi.container.docker.Compress')
    @patch('kiwi.container.oci.Command.run')
    def test_pack_image_to_file(self, mock_command, mock_compress):
        docker = ContainerImageDocker('root_dir', {'container_name': 'foo/bar'})
        docker.oci_dir = 'kiwi_oci_dir'
        docker.pack_image_to_file('result.tar.xz')

        assert mock_command.call_args_list == [
            call(['rm', '-r', '-f', 'result.tar']),
            call([
                'skopeo', 'copy', 'oci:kiwi_oci_dir/umoci_layout:latest',
                'docker-archive:result.tar:foo/bar:latest'
            ])
        ]
        mock_compress.assert_called_once_with('result.tar')



from mock import patch
from mock import call
from collections import namedtuple

from .test_helper import raises
import logging

from kiwi.logger import (
    LoggerSchedulerFilter,
    ColorFormatter,
    InfoFilter,
    DebugFilter,
    ErrorFilter,
    WarningFilter,
    log
)

from kiwi.exceptions import KiwiLogFileSetupFailed


class TestLoggerSchedulerFilter(object):
    def setup(self):
        self.scheduler_filter = LoggerSchedulerFilter()

    def test_filter(self):
        MyRecord = namedtuple(
            'MyRecord',
            'name'
        )
        ignorables = [
            'apscheduler.scheduler',
            'apscheduler.executors.default'
        ]
        for ignorable in ignorables:
            record = MyRecord(name=ignorable)
            assert self.scheduler_filter.filter(record) is False


class TestColorFormatter(object):
    def setup(self):
        self.color_formatter = ColorFormatter('%(levelname)s: %(message)s')

    @patch('logging.Formatter.format')
    def test_format(self, mock_format):
        MyRecord = namedtuple(
            'MyRecord',
            'levelname'
        )
        record = MyRecord(levelname='INFO')
        mock_format.return_value = 'message'
        self.color_formatter.format(record)
        assert 'message' in self.color_formatter.format(record)


class TestInfoFilter(object):
    def setup(self):
        self.info_filter = InfoFilter()

    def test_filter(self):
        MyRecord = namedtuple(
            'MyRecord',
            'levelno'
        )
        record = MyRecord(levelno=logging.INFO)
        assert self.info_filter.filter(record) is True


class TestDebugFilter(object):
    def setup(self):
        self.debug_filter = DebugFilter()

    def test_filter(self):
        MyRecord = namedtuple(
            'MyRecord',
            'levelno'
        )
        record = MyRecord(levelno=logging.DEBUG)
        assert self.debug_filter.filter(record) is True


class TestErrorFilter(object):
    def setup(self):
        self.error_filter = ErrorFilter()

    def test_filter(self):
        MyRecord = namedtuple(
            'MyRecord',
            'levelno'
        )
        record = MyRecord(levelno=logging.ERROR)
        assert self.error_filter.filter(record) is True


class TestWarningFilter(object):
    def setup(self):
        self.error_filter = WarningFilter()

    def test_filter(self):
        MyRecord = namedtuple(
            'MyRecord',
            'levelno'
        )
        record = MyRecord(levelno=logging.WARNING)
        assert self.error_filter.filter(record) is True


class TestLogger(object):
    @patch('sys.stdout')
    def test_progress(self, mock_stdout):
        log.progress(50, 100, 'foo')
        mock_stdout.write.assert_called_once_with(
            '\rfoo: [####################                    ] 50%'
        )
        mock_stdout.flush.assert_called_once_with()

    def test_progress_raise(self):
        assert log.progress(50, 0, 'foo') is None

    @patch('logging.FileHandler')
    def test_set_logfile(self, mock_file_handler):
        log.set_logfile('logfile')
        mock_file_handler.assert_called_once_with(
            filename='logfile', encoding='utf-8'
        )

    @patch('kiwi.logger.ColorFormatter')
    def test_set_color_format(self, mock_color_format):
        log.set_color_format()
        assert sorted(mock_color_format.call_args_list) == [
            call(
                '$COLOR[ %(levelname)-8s]: %(asctime)-8s | %(message)s',
                '%H:%M:%S'
            ),
            call(
                '$COLOR[ %(levelname)-8s]: %(asctime)-8s | %(message)s',
                '%H:%M:%S'
            ),
            call(
                '$LIGHTCOLOR[ %(levelname)-8s]: %(asctime)-8s | %(message)s',
                '%H:%M:%S'
            )
        ]

    @raises(KiwiLogFileSetupFailed)
    @patch('logging.FileHandler')
    def test_set_logfile_raise(self, mock_file_handler):
        mock_file_handler.side_effect = KiwiLogFileSetupFailed
        log.set_logfile('logfile')

    def test_getLogLevel(self):
        log.setLogLevel(42)
        assert log.getLogLevel() == 42



import sys
import mock
import os

from mock import patch

import kiwi

from .test_helper import argv_kiwi_tests

from kiwi.tasks.system_build import SystemBuildTask


class TestSystemBuildTask(object):
    def setup(self):
        sys.argv = [
            sys.argv[0], '--profile', 'vmxFlavour', 'system', 'build',
            '--description', '../data/description',
            '--target-dir', 'some-target'
        ]
        self.abs_target_dir = os.path.abspath('some-target')

        kiwi.tasks.system_build.Privileges = mock.Mock()
        kiwi.tasks.system_build.Path = mock.Mock()

        kiwi.tasks.system_build.Help = mock.Mock(
            return_value=mock.Mock()
        )

        self.manager = mock.Mock()
        self.system_prepare = mock.Mock()
        self.system_prepare.setup_repositories = mock.Mock(
            return_value=self.manager
        )

        self.runtime_checker = mock.Mock()
        kiwi.tasks.base.RuntimeChecker = mock.Mock(
            return_value=self.runtime_checker
        )

        kiwi.tasks.system_build.SystemPrepare = mock.Mock(
            return_value=self.system_prepare
        )

        self.setup = mock.Mock()
        kiwi.tasks.system_build.SystemSetup = mock.Mock(
            return_value=self.setup
        )

        self.profile = mock.Mock()
        kiwi.tasks.system_build.Profile = mock.Mock(
            return_value=self.profile
        )

        self.result = mock.Mock()
        self.builder = mock.MagicMock()
        self.builder.create = mock.Mock(
            return_value=self.result
        )
        kiwi.tasks.system_build.ImageBuilder = mock.Mock(
            return_value=self.builder
        )

        self.task = SystemBuildTask()

    def teardown(self):
        sys.argv = argv_kiwi_tests

    def _init_command_args(self):
        self.task.command_args = {}
        self.task.command_args['help'] = False
        self.task.command_args['build'] = False
        self.task.command_args['--description'] = '../data/description'
        self.task.command_args['--target-dir'] = 'some-target'
        self.task.command_args['--obs-repo-internal'] = None
        self.task.command_args['--set-repo'] = None
        self.task.command_args['--add-repo'] = []
        self.task.command_args['--add-package'] = []
        self.task.command_args['--delete-package'] = []
        self.task.command_args['--ignore-repos'] = False
        self.task.command_args['--set-container-derived-from'] = None
        self.task.command_args['--set-container-tag'] = None

    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_build(self, mock_log):
        self._init_command_args()
        self.task.command_args['build'] = True
        self.task.process()
        self.runtime_checker.check_docker_tool_chain_installed.assert_called_once_with()
        self.runtime_checker.check_image_include_repos_http_resolvable.assert_called_once_with()
        self.runtime_checker.check_target_directory_not_in_shared_cache.assert_called_once_with(self.abs_target_dir)
        self.runtime_checker.check_repositories_configured.assert_called_once_with()
        self.system_prepare.setup_repositories.assert_called_once_with()
        self.system_prepare.install_bootstrap.assert_called_once_with(
            self.manager
        )
        self.system_prepare.install_system.assert_called_once_with(
            self.manager
        )
        self.setup.import_shell_environment.assert_called_once_with(
            self.profile
        )
        self.setup.import_description.assert_called_once_with()
        self.setup.import_overlay_files.assert_called_once_with()
        self.setup.import_repositories_marked_as_imageinclude.assert_called_once_with()
        self.setup.call_config_script.assert_called_once_with()
        self.setup.import_image_identifier.assert_called_once_with()
        self.setup.setup_groups.assert_called_once_with()
        self.setup.setup_users.assert_called_once_with()
        self.setup.setup_keyboard_map.assert_called_once_with()
        self.setup.setup_locale.assert_called_once_with()
        self.setup.setup_timezone.assert_called_once_with()
        self.system_prepare.pinch_system.assert_called_once_with(
            manager=self.manager, force=True
        )
        self.setup.call_image_script.assert_called_once_with()
        self.builder.create.assert_called_once_with()
        self.result.print_results.assert_called_once_with()
        self.result.dump.assert_called_once_with(
            os.sep.join([self.abs_target_dir, 'kiwi.result'])
        )

    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_build_add_package(self, mock_log):
        self._init_command_args()
        self.task.command_args['--add-package'] = ['vim']
        self.task.process()
        self.system_prepare.setup_repositories.assert_called_once_with()
        self.system_prepare.install_packages.assert_called_once_with(
            self.manager, ['vim']
        )

    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_update_delete_package(self, mock_log):
        self._init_command_args()
        self.task.command_args['--delete-package'] = ['vim']
        self.task.process()
        self.system_prepare.setup_repositories.assert_called_once_with()
        self.system_prepare.delete_packages.assert_called_once_with(
            self.manager, ['vim']
        )

    @patch('kiwi.xml_state.XMLState.set_container_config_tag')
    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_build_prepare_stage_set_container_tag(
        self, mock_log, mock_set_container_tag
    ):
        self._init_command_args()
        self.task.command_args['--set-container-tag'] = 'new_tag'
        self.task.process()
        mock_set_container_tag.assert_called_once_with(
            'new_tag'
        )

    @patch('kiwi.xml_state.XMLState.set_derived_from_image_uri')
    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_build_prepare_stage_set_derived_from_uri(
        self, mock_log, mock_set_derived_from_uri
    ):
        self._init_command_args()
        self.task.command_args['--set-container-derived-from'] = 'file:///new'
        self.task.process()
        mock_set_derived_from_uri.assert_called_once_with(
            'file:///new'
        )

    @patch('kiwi.xml_state.XMLState.set_repository')
    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_build_prepare_stage_set_repo(
        self, mock_log, mock_set_repo
    ):
        self._init_command_args()
        self.task.command_args['--set-repo'] = 'http://example.com,yast2,alias'
        self.task.process()
        mock_set_repo.assert_called_once_with(
            'http://example.com', 'yast2', 'alias', None
        )

    @patch('kiwi.xml_state.XMLState.add_repository')
    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_build_prepare_stage_add_repo(
        self, mock_log, mock_add_repo
    ):
        self._init_command_args()
        self.task.command_args['--add-repo'] = [
            'http://example.com,yast2,alias'
        ]
        self.task.process()
        mock_add_repo.assert_called_once_with(
            'http://example.com', 'yast2', 'alias', None
        )

    @patch('kiwi.logger.Logger.set_logfile')
    @patch('kiwi.xml_state.XMLState.translate_obs_to_ibs_repositories')
    def test_process_system_prepare_use_ibs_repos(
        self, mock_ibs_repo, mock_log
    ):
        self._init_command_args()
        self.task.command_args['--obs-repo-internal'] = True
        self.task.process()
        mock_ibs_repo.assert_called_once_with()

    @patch('kiwi.logger.Logger.set_logfile')
    @patch('kiwi.xml_state.XMLState.translate_obs_to_suse_repositories')
    @patch('os.path.exists')
    def test_process_system_prepare_use_suse_repos(
        self, mock_exists, mock_suse_repos, mock_log
    ):
        self._init_command_args()
        mock_exists.return_value = True
        self.task.process()
        mock_suse_repos.assert_called_once_with()

    def test_process_system_build_help(self):
        self._init_command_args()
        self.task.command_args['help'] = True
        self.task.command_args['build'] = True
        self.task.process()
        self.task.manual.show.assert_called_once_with(
            'kiwi::system::build'
        )

    @patch('kiwi.xml_state.XMLState.delete_repository_sections')
    @patch('kiwi.logger.Logger.set_logfile')
    def test_process_system_prepare_ignore_repos(
        self, mock_log, mock_delete_repos
    ):
        self._init_command_args()
        self.task.command_args['--ignore-repos'] = True
        self.task.process()
        mock_delete_repos.assert_called_once_with()



from mock import patch
import mock

from .test_helper import raises

from kiwi.solver.sat import Sat
from kiwi.exceptions import (
    KiwiSatSolverPluginError,
    KiwiSatSolverJobProblems,
    KiwiSatSolverJobError
)


class TestSat(object):
    @patch('importlib.import_module')
    def setup(self, mock_import_module):
        self.sat = Sat()
        self.solver = mock.MagicMock()
        self.transaction = mock.Mock()
        self.transaction.newpackages = mock.Mock(
            return_value=[mock.Mock()]
        )
        self.selection = mock.Mock()
        self.solver.transaction = mock.Mock(
            return_value=self.transaction
        )
        self.sat.pool.Solver = mock.Mock(
            return_value=self.solver
        )
        self.sat.pool.select = mock.Mock(
            return_value=self.selection
        )
        mock_import_module.assert_called_once_with('solv')

    @patch('importlib.import_module')
    @raises(KiwiSatSolverPluginError)
    def test_setup_no_sat_plugin(self, mock_import_module):
        mock_import_module.side_effect = Exception
        Sat()

    def test_add_repository(self):
        solver_repository = mock.Mock()
        solver_repository.uri.uri = 'some-uri'
        solvable = mock.Mock()
        solver_repository.create_repository_solvable.return_value = solvable
        pool_repository = mock.Mock()
        self.sat.pool.add_repo.return_value = pool_repository

        self.sat.add_repository(solver_repository)

        solver_repository.create_repository_solvable.assert_called_once_with()
        self.sat.pool.add_repo.assert_called_once_with('some-uri')
        pool_repository.add_solv.assert_called_once_with(solvable)
        self.sat.pool.addfileprovides.assert_called_once_with()
        self.sat.pool.createwhatprovides.assert_called_once_with()

    @raises(KiwiSatSolverJobProblems)
    @patch.object(Sat, '_setup_jobs')
    def test_solve_has_problems(self, mock_setup_jobs):
        packages = ['vim']
        problem = mock.Mock()
        problem.id = 42
        info = mock.Mock()
        info.problemstr = mock.Mock(
            return_value='some-problem'
        )
        findproblemrule = mock.Mock()
        findproblemrule.info = mock.Mock(
            return_value=info
        )
        problem.findproblemrule.return_value = findproblemrule

        option = mock.Mock()
        option.str = mock.Mock(
            return_value='some-option'
        )
        solution = mock.Mock()
        solution.id = 42
        solution.elements = mock.Mock(
            return_value=[option]
        )
        problem.solutions.return_value = [solution]
        self.solver.solve = mock.Mock(
            return_value=[problem]
        )
        self.sat.solve(packages)

    @patch('kiwi.logger.log.info')
    def test_solve_package_not_found_and_skipped(self, mock_log_info):
        packages = ['vim']
        self.solver.solve = mock.Mock(
            return_value=None
        )
        self.sat.solv.Selection.SELECTION_PROVIDES = 0
        self.selection.flags = mock.Mock(
            return_value=0
        )
        self.selection.isempty = mock.Mock(
            return_value=True
        )
        self.sat.solve(packages, skip_missing=True)
        mock_log_info.assert_called_once_with(
            '--> Package vim not found: skipped'
        )

    @raises(KiwiSatSolverJobError)
    def test_solve_package_not_found_raises(self):
        packages = ['vim']
        self.solver.solve = mock.Mock(
            return_value=None
        )
        self.selection.isempty = mock.Mock(
            return_value=True
        )
        self.sat.solve(packages)

    def test_solve(self):
        packages = ['vim']
        self.solver.solve = mock.Mock(
            return_value=None
        )
        self.selection.isempty = mock.Mock(
            return_value=False
        )
        self.selection.jobs = mock.Mock(
            return_value=packages
        )
        self.sat.solve(packages)
        self.solver.solve.assert_called_once_with(['vim'])
        self.solver.transaction.assert_called_once_with()

    @patch('kiwi.logger.log.info')
    def test_solve_with_capabilities(self, mock_log_info):
        packages = ['kernel-base']
        self.solver.solve = mock.Mock(
            return_value=None
        )
        self.sat.solv.Selection.SELECTION_PROVIDES = 1
        self.selection.flags = mock.Mock(
            return_value=1
        )
        self.selection.isempty = mock.Mock(
            return_value=False
        )
        self.selection.jobs = mock.Mock(
            return_value=packages
        )
        self.sat.solve(packages)
        mock_log_info.assert_called_once_with(
            '--> Using capability match for kernel-base'
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os

# project
from kiwi.container.oci import ContainerImageOCI
from kiwi.path import Path
from kiwi.command import Command
from kiwi.utils.compress import Compress


class ContainerImageDocker(ContainerImageOCI):
    """
     Create docker container from a root directory

     Attributes

     * :attr:`root_dir`
         root directory path name

     * :attr:`custom_args`
         representation of the containerconfig and its subsections
     """
    def pack_image_to_file(self, filename):
        """
        Packs the given oci image into the given filename.

        :param string filename: file name of the resulting packed image
        """
        docker_tarfile = filename.replace('.xz', '')
        oci_image = os.sep.join([
            self.oci_dir, ':'.join(['umoci_layout', self.container_tag])
        ])

        # make sure the target tar file does not exist
        # skopeo doesn't support force overwrite
        Path.wipe(docker_tarfile)
        Command.run(
            [
                'skopeo', 'copy', 'oci:{0}'.format(
                    oci_image
                ),
                'docker-archive:{0}:{1}:{2}'.format(
                    docker_tarfile, self.container_name, self.container_tag
                )
            ]
        )
        compressor = Compress(docker_tarfile)
        compressor.xz()



from mock import patch
from mock import call
import mock
import os

from .test_helper import raises, patch_open

from textwrap import dedent

from kiwi.exceptions import (
    KiwiVmdkToolsError,
    KiwiTemplateError
)

from kiwi.storage.subformat.vmdk import DiskFormatVmdk

from builtins import bytes


class TestDiskFormatVmdk(object):
    @patch('platform.machine')
    def setup(self, mock_machine):
        self.context_manager_mock = mock.Mock()
        self.file_mock = mock.Mock()
        self.enter_mock = mock.Mock()
        self.exit_mock = mock.Mock()
        self.enter_mock.return_value = self.file_mock
        setattr(self.context_manager_mock, '__enter__', self.enter_mock)
        setattr(self.context_manager_mock, '__exit__', self.exit_mock)
        mock_machine.return_value = 'x86_64'
        xml_data = mock.Mock()
        xml_data.get_name = mock.Mock(
            return_value='some-disk-image'
        )
        xml_data.get_displayname = mock.Mock(
            return_value=None
        )
        self.xml_state = mock.Mock()
        self.xml_state.xml_data = xml_data
        self.xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )

        self.xml_state.get_build_type_vmconfig_entries = mock.Mock(
            return_value=['custom entry 1', 'custom entry 2']
        )

        self.machine_setup = mock.Mock()
        self.xml_state.get_build_type_machine_section = mock.Mock(
            return_value=self.machine_setup
        )
        self.machine_setup.get_HWversion = mock.Mock(
            return_value='42'
        )
        self.machine_setup.get_guestOS = mock.Mock(
            return_value='suse'
        )
        self.machine_setup.get_memory = mock.Mock(
            return_value='4096'
        )
        self.machine_setup.get_ncpus = mock.Mock(
            return_value='2'
        )

        self.iso_setup = mock.Mock()
        self.xml_state.get_build_type_vmdvd_section = mock.Mock(
            return_value=self.iso_setup
        )
        self.iso_setup.get_controller = mock.Mock(
            return_value='ide'
        )
        self.iso_setup.get_id = mock.Mock(
            return_value='0'
        )

        self.network_setup = mock.Mock()
        self.xml_state.get_build_type_vmnic_section = mock.Mock(
            return_value=self.network_setup
        )
        self.network_setup.get_interface = mock.Mock(
            return_value='0'
        )
        self.network_setup.get_mac = mock.Mock(
            return_value='98:90:96:a0:3c:58'
        )
        self.network_setup.get_mode = mock.Mock(
            return_value='bridged'
        )
        self.network_setup.get_driver = mock.Mock(
            return_value='e1000'
        )

        self.disk_setup = mock.Mock()
        self.xml_state.get_build_type_vmdisk_section = mock.Mock(
            return_value=self.disk_setup
        )
        self.disk_setup.get_controller = mock.Mock(
            return_value='lsilogic'
        )
        self.disk_setup.get_id = mock.Mock(
            return_value='0'
        )

        self.disk_format = DiskFormatVmdk(
            self.xml_state, 'root_dir', 'target_dir'
        )
        self.vmdk_header_update = dedent('''
            encoding="UTF-8"
            dd-out
            ddb.toolsInstallType = "4"
            ddb.toolsVersion = "9350"
        ''').strip()
        self.vmdk_header_update = bytes(self.vmdk_header_update, 'utf-8')

        self.vmdk_settings_formatted = dedent('''
            #!/usr/bin/env vmware
            # kiwi generated VMware settings file
            config.version = "8"
            tools.syncTime = "true"
            uuid.action = "create"
            virtualHW.version = "42"
            displayName = "some-disk-image"
            guestOS = "suse"
            priority.grabbed = "normal"
            priority.ungrabbed = "normal"
            powerType.powerOff = "soft"
            powerType.powerOn  = "soft"
            powerType.suspend  = "soft"
            powerType.reset    = "soft"
            memsize = "4096"
            numvcpus = "2"
            scsi0.present = "true"
            scsi0.sharedBus = "none"
            scsi0.virtualDev = "lsilogic"
            scsi0:0.present = "true"
            scsi0:0.fileName = "target_dir/some-disk-image.x86_64-1.2.3.vmdk"
            scsi0:0.deviceType = "scsi-hardDisk"
            ethernet0.present = "true"
            ethernet0.allow64bitVmxnet = "true"
            ethernet0.addressType = "static"
            ethernet0.address = "98:90:96:a0:3c:58"
            ethernet0.virtualDev = "e1000"
            ethernet0.connectionType = "bridged"
            ide0:0.present = "true"
            ide0:0.deviceType = "cdrom-raw"
            ide0:0.autodetect = "true"
            ide0:0.startConnected = "true"
            usb.present = "true"
        ''').strip() + '\n'

        self.vmdk_settings = ''
        for entry in self.vmdk_settings_formatted.split('\n'):
            if entry:
                self.vmdk_settings += entry.strip() + '\n'

    def test_post_init(self):
        self.disk_format.post_init({'option': 'value'})
        assert self.disk_format.options == ['-o', 'option', 'value']

    def test_store_to_result(self):
        result = mock.Mock()
        self.disk_format.store_to_result(result)
        assert result.add.call_args_list == [
            call(
                compress=False,
                filename='target_dir/some-disk-image.x86_64-1.2.3.vmdk',
                key='disk_format_image',
                shasum=True,
                use_for_bundle=True
            ),
            call(
                compress=False,
                filename='target_dir/some-disk-image.x86_64-1.2.3.vmx',
                key='disk_format_machine_settings',
                shasum=False,
                use_for_bundle=True
            )
        ]

    @patch('kiwi.storage.subformat.vmdk.Command.run')
    @patch('os.path.exists')
    @patch('kiwi.logger.log.warning')
    @patch_open
    def test_create_image_format_skip_descriptor_update(
        self, mock_open, mock_log_warn, mock_exists, mock_command
    ):
        mock_exists.return_value = False
        self.disk_format.create_image_format()
        mock_command.assert_called_once_with(
            [
                'qemu-img', 'convert', '-f', 'raw',
                'target_dir/some-disk-image.x86_64-1.2.3.raw', '-O', 'vmdk',
                'target_dir/some-disk-image.x86_64-1.2.3.vmdk'
            ]
        )
        assert mock_log_warn.called

    @raises(KiwiVmdkToolsError)
    @patch('kiwi.storage.subformat.vmdk.Command.run')
    @patch('os.path.exists')
    def test_create_image_format_invalid_tools_version(
        self, mock_exists, mock_command
    ):
        command = mock.Mock()
        command.output = 'bogus-version'
        mock_command.return_value = command
        mock_exists.return_value = True
        self.disk_format.create_image_format()

    @raises(KiwiTemplateError)
    @patch('kiwi.storage.subformat.vmdk.VmwareSettingsTemplate.get_template')
    @patch('kiwi.storage.subformat.vmdk.Command.run')
    @patch('os.path.exists')
    @patch_open
    def test_create_image_format_template_error(
        self, mock_open, mock_exists, mock_command, mock_get_template
    ):
        qemu_img_result = mock.Mock()
        vmtoolsd_result = mock.Mock()
        vmtoolsd_result.output = \
            'VMware Tools daemon, version 9.4.6.33107 (build-0815)'
        dd_result = mock.Mock()
        dd_result.output = 'dd-out\0\0\0\0\0'

        command_results = [
            dd_result, vmtoolsd_result, qemu_img_result
        ]

        def side_effect(arg):
            return command_results.pop()

        mock_command.side_effect = side_effect
        template = mock.Mock()
        mock_get_template.return_value = template
        template.substitute.side_effect = Exception
        self.disk_format.create_image_format()

    @patch('kiwi.storage.subformat.vmdk.Command.run')
    @patch('os.path.exists')
    @patch_open
    def test_create_image_format(
        self, mock_open, mock_exists, mock_command
    ):
        qemu_img_result = mock.Mock()
        vmtoolsd_result = mock.Mock()
        vmtoolsd_result.output = \
            'VMware Tools daemon, version 9.4.6.33107 (build-0815)'
        dd_result = mock.Mock()
        dd_result.output = 'dd-out\0\0\0\0\0'

        command_results = [
            dd_result, vmtoolsd_result, qemu_img_result
        ]

        def side_effect(arg):
            return command_results.pop()

        mock_command.side_effect = side_effect
        mock_open.return_value = self.context_manager_mock
        mock_exists.return_value = True

        self.disk_format.create_image_format()

        assert mock_open.call_args_list == [
            call('target_dir/some-disk-image.x86_64-1.2.3.vmdk', 'ab'),
            call('target_dir/some-disk-image.x86_64-1.2.3.vmx', 'w')
        ]
        assert self.file_mock.write.call_args_list[0] == call(
            self.vmdk_header_update
        )
        assert self.file_mock.write.call_args_list[1] == call(
            self.vmdk_settings
        )
        assert self.file_mock.seek.call_args_list == [
            call(512, 0), call(0, 2)
        ]

    @patch('kiwi.storage.subformat.vmdk.Command.run')
    @patch('os.path.exists')
    @patch_open
    def test_create_image_format_encoding_exists(
        self, mock_open, mock_exists, mock_command
    ):
        qemu_img_result = mock.Mock()
        vmtoolsd_result = mock.Mock()
        vmtoolsd_result.output = \
            'VMware Tools daemon, version 9.4.6.33107 (build-0815)'
        dd_result = mock.Mock()
        dd_result.output = 'encoding="UTF-8"\ndd-out\0\0\0\0\0'

        command_results = [
            dd_result, vmtoolsd_result, qemu_img_result
        ]

        def side_effect(arg):
            return command_results.pop()

        mock_command.side_effect = side_effect
        mock_open.return_value = self.context_manager_mock
        mock_exists.return_value = True

        self.disk_format.create_image_format()

        assert mock_open.call_args_list == [
            call('target_dir/some-disk-image.x86_64-1.2.3.vmdk', 'ab'),
            call('target_dir/some-disk-image.x86_64-1.2.3.vmx', 'w')
        ]
        assert self.file_mock.write.call_args_list[0] == call(
            self.vmdk_header_update
        )
        assert self.file_mock.write.call_args_list[1] == call(
            self.vmdk_settings
        )
        assert self.file_mock.write.call_args_list[2] == call(
            'custom entry 1' + os.linesep
        )
        assert self.file_mock.write.call_args_list[3] == call(
            'custom entry 2' + os.linesep
        )
        assert self.file_mock.seek.call_args_list == [
            call(512, 0), call(0, 2)
        ]



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
from collections import namedtuple
import platform
from pkg_resources import resource_filename

# project
from .version import __githash__


class Defaults(object):
    """
    Default values and export methods

    Attributes

    * :attr:`defaults`
        dict of default profile values

    * :attr:`profile_key_list`
        list of profile keys to import into an instance of Profile
    """
    def __init__(self):
        self.defaults = {
            # alignment in bytes
            'kiwi_align': 1048576,
            # start sector number
            'kiwi_startsector': 2048,
            # sectorsize in bytes
            'kiwi_sectorsize': 512,
            # inode size in bytes for inode based filesystems
            'kiwi_inode_size': 256,
            # inode ratio for inode based filesystems
            'kiwi_inode_ratio': 16384,
            # minimum inode number for inode based filesystems
            'kiwi_min_inodes': 20000,
            # kiwi git revision
            'kiwi_revision': __githash__
        }
        self.profile_key_list = [
            'kiwi_align',
            'kiwi_startsector',
            'kiwi_sectorsize',
            'kiwi_revision'
        ]

    @classmethod
    def is_obs_worker(self):
        # the presence of /.buildenv on the build host indicates
        # we are building inside of the open buildservice
        return os.path.exists('/.buildenv')

    @classmethod
    def get_s390_disk_block_size(self):
        """
        The default block size for s390 storage disks
        """
        return '4096'

    @classmethod
    def get_s390_disk_type(self):
        """
        The default disk type for s390 storage disks
        """
        return 'CDL'

    @classmethod
    def get_solvable_location(self):
        """
        The directory to store SAT solvables for repositories.
        The solvable files are used to perform package
        dependency and metadata resolution
        """
        return '/var/tmp/kiwi/satsolver'

    @classmethod
    def get_shared_cache_location(self):
        """
        The shared location is a directory which shares data from
        the image buildsystem host with the image root system.
        The location is returned as an absolute path stripped off
        by the leading '/'. This is because the path is transparently
        used on the host /<cache-dir> and inside of the image
        imageroot/<cache-dir>
        """
        from .cli import Cli
        return os.path.abspath(os.path.normpath(
            Cli().get_global_args().get('--shared-cache-dir')
        )).lstrip(os.sep)

    @classmethod
    def get_failsafe_kernel_options(self):
        """
        Failsafe boot kernel options

        :return: kernel options list
        :rtype: list
        """
        return ' '.join(
            [
                'ide=nodma',
                'apm=off',
                'noresume',
                'edd=off',
                'powersaved=off',
                'nohz=off',
                'highres=off',
                'processsor.max+cstate=1',
                'nomodeset',
                'x11failsafe'
            ]
        )

    @classmethod
    def get_video_mode_map(self):
        """
        Implements video mode map

        Assign a tuple to each kernel vesa hex id for each of the
        supported bootloaders

        :return: video type map
        :rtype: dict
        """
        video_type = namedtuple(
            'video_type', ['grub2', 'isolinux']
        )
        return {
            '0x301': video_type(grub2='640x480', isolinux='640 480'),
            '0x310': video_type(grub2='640x480', isolinux='640 480'),
            '0x311': video_type(grub2='640x480', isolinux='640 480'),
            '0x312': video_type(grub2='640x480', isolinux='640 480'),
            '0x303': video_type(grub2='800x600', isolinux='800 600'),
            '0x313': video_type(grub2='800x600', isolinux='800 600'),
            '0x314': video_type(grub2='800x600', isolinux='800 600'),
            '0x315': video_type(grub2='800x600', isolinux='800 600'),
            '0x305': video_type(grub2='1024x768', isolinux='1024 768'),
            '0x316': video_type(grub2='1024x768', isolinux='1024 768'),
            '0x317': video_type(grub2='1024x768', isolinux='1024 768'),
            '0x318': video_type(grub2='1024x768', isolinux='1024 768'),
            '0x307': video_type(grub2='1280x1024', isolinux='1280 1024'),
            '0x319': video_type(grub2='1280x1024', isolinux='1280 1024'),
            '0x31a': video_type(grub2='1280x1024', isolinux='1280 1024'),
            '0x31b': video_type(grub2='1280x1024', isolinux='1280 1024'),
        }

    @classmethod
    def get_default_video_mode(self):
        """
        Implements 800x600 default video mode

        :return: video mode name
        :rtype: string
        """
        return '0x303'

    @classmethod
    def get_grub_basic_modules(self, multiboot):
        """
        Implements list of basic grub modules

        :param bool multiboot: grub multiboot mode

        :return: module name
        :rtype: string
        """
        modules = [
            'ext2',
            'iso9660',
            'linux',
            'echo',
            'configfile',
            'search_label',
            'search_fs_file',
            'search',
            'search_fs_uuid',
            'ls',
            'normal',
            'gzio',
            'png',
            'fat',
            'gettext',
            'font',
            'minicmd',
            'gfxterm',
            'gfxmenu',
            'video',
            'video_fb',
            'xfs',
            'btrfs',
            'lvm',
            'test'
        ]
        if multiboot:
            modules.append('multiboot')
        return modules

    @classmethod
    def get_grub_efi_modules(self, multiboot=False):
        """
        Implements list of grub efi modules

        :param bool multiboot: grub multiboot mode

        :return: module name
        :rtype: string
        """
        host_architecture = platform.machine()
        modules = Defaults.get_grub_basic_modules(multiboot) + [
            'part_gpt',
            'part_msdos',
            'efi_gop'
        ]
        if host_architecture == 'x86_64':
            modules += [
                'efi_uga',
                'linuxefi'
            ]
        return modules

    @classmethod
    def get_grub_bios_modules(self, multiboot=False):
        """
        Implements list of grub bios modules

        :param bool multiboot: grub multiboot mode

        :return: module name
        :rtype: string
        """
        modules = Defaults.get_grub_basic_modules(multiboot) + [
            'part_gpt',
            'part_msdos',
            'biosdisk',
            'vga',
            'vbe',
            'chain',
            'boot'
        ]
        return modules

    @classmethod
    def get_grub_ofw_modules(self):
        """
        Implements list of grub ofw modules (ppc)

        :return: module name
        :rtype: string
        """
        modules = Defaults.get_grub_basic_modules(multiboot=False) + [
            'part_gpt',
            'part_msdos',
            'boot'
        ]
        return modules

    @classmethod
    def get_grub_path(self, lookup_path):
        """
        Depending on the distribution grub could be installed below
        a grub2 or grub directory. Therefore this information needs
        to be dynamically looked up

        :param string lookup_path: path name

        :return: grub2 install directory path name
        :rtype: string
        """
        for grub_name in ['grub2', 'grub']:
            grub_path = lookup_path + '/' + grub_name
            if os.path.exists(grub_path):
                return grub_path

    @classmethod
    def get_preparer(self):
        """
        Implements ISO preparer name

        :return: name
        :rtype: string
        """
        return 'KIWI - http://suse.github.com/kiwi'

    @classmethod
    def get_publisher(self):
        """
        Implements ISO publisher name

        :return: name
        :rtype: string
        """
        return 'SUSE LINUX GmbH'

    @classmethod
    def get_shim_name(self):
        return 'shim.efi'

    @classmethod
    def get_signed_grub_name(self):
        return 'grub.efi'

    @classmethod
    def get_default_volume_group_name(self):
        """
        Implements default LVM volume group name

        :return: name
        :rtype: string
        """
        return 'systemVG'

    @classmethod
    def get_min_volume_mbytes(self):
        """
        Implements default minimum LVM volume size in mbytes

        :return: mbsize
        :rtype: int
        """
        return 30

    @classmethod
    def get_default_boot_mbytes(self):
        """
        Implements default boot partition size in mbytes

        :return: mbsize
        :rtype: int
        """
        return 300

    @classmethod
    def get_default_efi_boot_mbytes(self):
        """
        Implements default EFI partition size in mbytes

        :return: mbsize
        :rtype: int
        """
        return 20

    @classmethod
    def get_recovery_spare_mbytes(self):
        """
        Implements spare size of recovery partition in mbytes

        :return: mbsize
        :rtype: int
        """
        return 300

    @classmethod
    def get_default_legacy_bios_mbytes(self):
        """
        Implements default size of bios_grub partition in mbytes

        :return: mbsize
        :rtype: int
        """
        return 2

    @classmethod
    def get_default_prep_mbytes(self):
        """
        Implements default size of prep partition in mbytes

        :return: mbsize
        :rtype: int
        """
        return 8

    @classmethod
    def get_disk_format_types(self):
        """
        Implements supported disk format types

        :return: disk types
        :rtype: list
        """
        return [
            'gce', 'qcow2', 'vmdk', 'vmx', 'vhd',
            'vhdfixed', 'vdi', 'vagrant.libvirt.box'
        ]

    @classmethod
    def get_firmware_types(self):
        """
        Implements supported architecture specific firmware types

        :return: firmware types
        :rtype: dict
        """
        return {
            'x86_64': ['efi', 'uefi', 'bios', 'ec2hvm', 'ec2'],
            'i586': ['bios'],
            'i686': ['bios'],
            'aarch64': ['efi', 'uefi'],
            'arm64': ['efi', 'uefi'],
            'armv5el': ['efi', 'uefi'],
            'armv5tel': ['efi', 'uefi'],
            'armv6hl': ['efi', 'uefi'],
            'armv6l': ['efi', 'uefi'],
            'armv7hl': ['efi', 'uefi'],
            'armv7l': ['efi', 'uefi'],
            'ppc': ['ofw'],
            'ppc64': ['ofw', 'opal'],
            'ppc64le': ['ofw', 'opal'],
            's390': [],
            's390x': []
        }

    @classmethod
    def get_default_firmware(self, arch):
        """
        Implements default firmware for specified architecture

        :param string arch: platform.machine

        :return: firmware
        :rtype: string
        """
        default_firmware = {
            'x86_64': 'bios',
            'i586': 'bios',
            'i686': 'bios',
            'ppc': 'ofw',
            'ppc64': 'ofw',
            'ppc64le': 'ofw',
            'arm64': 'efi',
            'armv5el': 'efi',
            'armv5tel': 'efi',
            'armv6hl': 'efi',
            'armv6l': 'efi',
            'armv7hl': 'efi',
            'armv7l': 'efi'
        }
        if arch in default_firmware:
            return default_firmware[arch]

    @classmethod
    def get_efi_capable_firmware_names(self):
        """
        Implements list of EFI capable firmware names. These are
        those for which kiwi supports the creation of an EFI
        bootable disk image

        :return: firmware names
        :rtype: list
        """
        return ['efi', 'uefi']

    @classmethod
    def get_ec2_capable_firmware_names(self):
        """
        Implements list of EC2 capable firmware names. These are
        those for which kiwi supports the creation of disk images
        bootable within the Amazon EC2 public cloud

        :return: firmware names
        :rtype: list
        """
        return ['ec2', 'ec2hvm']

    @classmethod
    def get_efi_module_directory_name(self, arch):
        """
        Implements architecture specific EFI directory name which
        stores the EFI binaries for the desired architecture.

        :param string arch: platform.machine

        :return: directory name
        :rtype: string
        """
        default_module_directory_names = {
            'x86_64': 'x86_64-efi',

            # There is no dedicated xen architecture but there are
            # modules provided for xen. Thus we treat it as an
            # architecture
            'x86_64_xen': 'x86_64-xen',

            'aarch64': 'arm64-efi',
            'arm64': 'arm64-efi',
            'armv5el': 'arm-efi',
            'armv5tel': 'arm-efi',
            'armv6l': 'arm-efi',
            'armv7l': 'arm-efi'
        }
        if arch in default_module_directory_names:
            return default_module_directory_names[arch]

    @classmethod
    def get_efi_image_name(self, arch):
        """
        Implements architecture specific EFI boot binary name

        :param string arch: platform.machine

        :return: name
        :rtype: string
        """
        default_efi_image_names = {
            'x86_64': 'bootx64.efi',
            'aarch64': 'bootaa64.efi',
            'arm64': 'bootaa64.efi',
            'armv5el': 'bootarm.efi',
            'armv5tel': 'bootarm.efi',
            'armv6l': 'bootarm.efi',
            'armv7l': 'bootarm.efi'
        }
        if arch in default_efi_image_names:
            return default_efi_image_names[arch]

    @classmethod
    def get_default_boot_timeout_seconds(self):
        """
        Implements default boot timeout in seconds

        :return: seconds
        :rtype: int
        """
        return 10

    @classmethod
    def get_default_inode_size(self):
        """
        Implements default size of inodes in bytes. This is only
        relevant for inode based filesystems

        :return: bytesize
        :rtype: int
        """
        return 256

    @classmethod
    def get_archive_image_types(self):
        """
        Implements list of supported archive image types

        :return: archive names
        :rtype: list
        """
        return ['tbz']

    @classmethod
    def get_container_image_types(self):
        """
        Implements list of supported container image types

        :return: container names
        :rtype: list
        """
        return ['docker', 'oci']

    @classmethod
    def get_filesystem_image_types(self):
        """
        Implements list of supported filesystem image types

        :return: filesystem names
        :rtype: list
        """
        return [
            'ext2', 'ext3', 'ext4', 'btrfs', 'squashfs',
            'xfs', 'fat16', 'fat32'
        ]

    @classmethod
    def get_live_iso_types(self):
        """
        Implements list of supported live ISO image types

        :return: live iso names
        :rtype: list
        """
        return {
            'overlay': 'squashfs',
            'clic': 'clicfs'
        }

    @classmethod
    def get_live_iso_client_parameters(self):
        """
        Implements parameters to setup the overlay filesystem used
        for the live ISO image. Each supported overlay filesystem
        needs the information about the target block device, the
        copy on write device and the used kernel union filesystem
        name

        :return: union client parameters
        :rtype: dict
        """
        return {
            'overlay': ['loop', 'tmpfs', 'overlay'],
            'clic': ['/dev/ram1', '/dev/ram1', 'clicfs']
        }

    @classmethod
    def get_default_live_iso_type(self):
        """
        Implements default live iso union type

        :return: live iso type
        :rtype: string
        """
        return 'overlay'

    @classmethod
    def get_disk_image_types(self):
        """
        Implements supported disk image types

        :return: disk image type names
        :rtype: list
        """
        return ['oem', 'vmx']

    @classmethod
    def get_live_image_types(self):
        """
        Implements supported live image types

        :return: live image type names
        :rtype: list
        """
        return ['iso']

    @classmethod
    def get_network_image_types(self):
        """
        Implements supported pxe image types

        :return: pxe image type names
        :rtype: list
        """
        return ['pxe']

    @classmethod
    def get_boot_image_description_path(self):
        """
        Implements bootloader path for ISO images

        :return: directory path
        :rtype: string
        """
        return Defaults.project_file('boot/arch/' + platform.machine())

    @classmethod
    def get_boot_image_strip_file(self):
        """
        Implements file path to bootloader strip metadata.
        This file contains information about the files and directories
        automatically striped out from the kiwi initrd

        :return: file path
        :rtype: string
        """
        return Defaults.project_file('config/strip.xml')

    @classmethod
    def get_schema_file(self):
        """
        Implements file path to kiwi RNG schema

        :return: file path
        :rtype: string
        """
        return Defaults.project_file('schema/kiwi.rng')

    @classmethod
    def get_common_functions_file(self):
        """
        Implements file path to bootloader functions metadata.
        This file contains bash functions used in the boot code
        from the kiwi initrd

        :return: file path
        :rtype: string
        """
        return Defaults.project_file('config/functions.sh')

    @classmethod
    def get_xsl_stylesheet_file(self):
        """
        Implements file path to kiwi XSLT style sheets

        :return: file path
        :rtype: string
        """
        return Defaults.project_file('xsl/master.xsl')

    @classmethod
    def project_file(self, filename):
        """
        Implements python module base directory search path
        The method uses the resource_filename method to identify
        files and directories from the application

        :param string filename: relative project file

        :return: full qualified filename
        :rtype: string
        """
        return resource_filename('kiwi', filename)

    @classmethod
    def get_imported_root_image(self, root_dir):
        """
        Returns the path of the image used to import a root during
        the prepare task. This is the filename expected to be
        found during the create step if derived_from attribute is
        present.

        :param string root_dir: is the root directory of the current build

        :return: file name of the image
        :rtype: string
        """
        return os.sep.join([root_dir, 'image', 'imported_root'])

    def get(self, key):
        """
        Implements get method for profile elements

        :param string key: profile keyname

        :return: key value
        :rtype: string
        """
        if key in self.defaults:
            return self.defaults[key]

    def to_profile(self, profile):
        """
        Implements method to add list of profile keys and their values
        to the specified instance of a Profile class

        :param object profile: Profile instance
        """
        for key in sorted(self.profile_key_list):
            profile.add(key, self.get(key))



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
from textwrap import dedent

# project
from .xml_description import XMLDescription
from .xml_state import XMLState
from .system.uri import Uri
from .defaults import Defaults
from .path import Path
from .exceptions import (
    KiwiRuntimeError
)


class RuntimeChecker(object):
    """
    Implements build consistency checks at runtime

    The schema of an image description covers structure and syntax of
    the provided data. The RuntimeChecker provides methods to perform
    further semantic checks which allows to recognize potential build
    or boot problems early.

    * :attr:`xml_state`
        Instance of XMLState
    """
    def __init__(self, xml_state):
        self.xml_state = xml_state

    def check_repositories_configured(self):
        """
        Verify that that there are repositories configured
        """
        if not self.xml_state.get_repository_sections():
            raise KiwiRuntimeError(
                'No repositories configured'
            )

    def check_image_include_repos_http_resolvable(self):
        """
        Verify that all repos marked with the imageinclude attribute
        can be resolved into a http based web URL
        """

        message = dedent('''\n
            Repository: %s is not publicly available.
            Therefore it can't be included into the system image
            repository configuration. Please check the setup of
            the <imageinclude> attribute for this repository.
        ''')

        repository_sections = self.xml_state.get_repository_sections()
        for xml_repo in repository_sections:
            repo_marked_for_image_include = xml_repo.get_imageinclude()

            if repo_marked_for_image_include:
                repo_source = xml_repo.get_source().get_path()
                repo_type = xml_repo.get_type()
                uri = Uri(repo_source, repo_type)
                if not uri.is_remote() or Defaults.is_obs_worker():
                    raise KiwiRuntimeError(message % repo_source)

    def check_target_directory_not_in_shared_cache(self, target_dir):
        """
        The target directory must be outside of the kiwi shared cache
        directory in order to avoid busy mounts because kiwi bind mounts
        the cache directory into the image root tree to access host
        caching information

        :param string target_dir: path name
        """

        message = dedent('''\n
            Target directory %s conflicts with kiwi's shared cache
            directory %s. This is going to create a busy loop mount.
            Please choose another target directory.
        ''')

        shared_cache_location = Defaults.get_shared_cache_location()
        absolute_target_dir = os.path.abspath(
            os.path.normpath(target_dir)
        ).replace('//', '/')
        if absolute_target_dir.startswith('/' + shared_cache_location):
            raise KiwiRuntimeError(
                message % (target_dir, shared_cache_location)
            )

    def check_volume_setup_has_no_root_definition(self):
        """
        The root volume in a systemdisk setup is handled in a special
        way. It is not allowed to setup a custom name or mountpoint for
        the root volume. Therefore the size of the root volume can be
        setup via the @root volume name. This check looks up the volume
        setup and searches if there is a configuration for the '/'
        mountpoint which would cause the image build to fail
        """
        message = dedent('''\n
            Volume setup for "/" found. The size of the root volume
            must be specified via the @root volume name like the
            following example shows:

            <volume name="@root" size="42G"/>

            A custom name or mountpoint for the root volume is not
            allowed.
        ''')
        for volume in self.xml_state.get_volumes():
            if volume.mountpoint == '/':
                raise KiwiRuntimeError(message)

    def check_docker_tool_chain_installed(self):
        """
        When creating docker images the tools umoci and skopeo are used
        in order to create docker compatible images. This check searches
        for those tools to be installed in the build system and fails if
        it can't find them
        """
        message = dedent('''\n
            Required tool {0} not found in caller environment

            Creation of docker images requires the tools umoci and skopeo
            to be installed on the build system. For SUSE based systems
            you can find the tools at:

            http://download.opensuse.org/repositories/Virtualization:/containers
        ''')
        if self.xml_state.get_build_type_name() == 'docker':
            for tool in ['umoci', 'skopeo']:
                if not Path.which(filename=tool, access_mode=os.X_OK):
                    raise KiwiRuntimeError(message.format(tool))

    def check_consistent_kernel_in_boot_and_system_image(self):
        """
        If a kiwi initrd is used, the kernel used to build the kiwi
        initrd and the kernel used in the system image must be the
        same in order to avoid an inconsistent boot setup
        """
        message = dedent('''\n
            Possible kernel mismatch between kiwi initrd and system image

            The selected '{0}' boot image kernel is '{1}'. However this
            kernel package was not explicitly listed in the package list
            of the system image. Please fixup your system image
            description:

            1) Add <package name="{1}"/> to your system XML description

            2) Inherit kernel from system description to initrd via
               the custom kernel profile:

               <type ... bootkernel="custom" .../>

               <packages type="image"/>
                   <package name="desired-kernel" bootinclude="true"/>
               </packages>
        ''')
        boot_image_reference = self.xml_state.build_type.get_boot()
        boot_kernel_package_name = None
        if boot_image_reference:
            if not boot_image_reference[0] == '/':
                boot_image_reference = os.sep.join(
                    [
                        Defaults.get_boot_image_description_path(),
                        boot_image_reference
                    ]
                )
            boot_config_file = os.sep.join(
                [boot_image_reference, 'config.xml']
            )
            if os.path.exists(boot_config_file):
                boot_description = XMLDescription(
                    description=boot_config_file,
                    derived_from=self.xml_state.xml_data.description_dir
                )
                boot_kernel_profile = \
                    self.xml_state.build_type.get_bootkernel()
                if not boot_kernel_profile:
                    boot_kernel_profile = 'std'
                boot_xml_state = XMLState(
                    boot_description.load(), [boot_kernel_profile]
                )
                kernel_package_sections = []
                for packages_section in boot_xml_state.xml_data.get_packages():
                    # lookup package sections matching kernel profile in kiwi
                    # boot description. By definition this must be a packages
                    # section with a single profile name whereas the default
                    # profile name is 'std'. The section itself must contain
                    # one matching kernel package name for the desired
                    # architecture
                    if packages_section.get_profiles() == boot_kernel_profile:
                        for package in packages_section.get_package():
                            kernel_package_sections.append(package)

                for package in kernel_package_sections:
                    if boot_xml_state.package_matches_host_architecture(package):
                        boot_kernel_package_name = package.get_name()

        if boot_kernel_package_name:
            # A kernel package name was found in the kiwi boot image
            # description. Let's check if this kernel is also used
            # in the system image
            image_package_names = self.xml_state.get_system_packages()
            if boot_kernel_package_name not in image_package_names:
                raise KiwiRuntimeError(
                    message.format(
                        self.xml_state.build_type.get_boot(),
                        boot_kernel_package_name
                    )
                )

    def check_boot_image_reference_correctly_setup(self):
        """
        If an initrd_system different from "kiwi" is selected for a
        vmx (simple disk) image build, it does not make sense to setup
        a reference to a kiwi boot image description, because no kiwi
        boot image will be built.
        """
        message = dedent('''\n
            Selected initrd_system is: {0}

            The boot attribute selected: '{1}' which is an initrd image
            used for the 'kiwi' initrd system. This boot image will not be
            used according to the selected initrd system. Please cleanup
            your image description:

            1) If the selected initrd system is correct, delete the
               obsolete boot attribute from the selected '{2}' build type

            2) If the kiwi initrd image should be used, make sure to
               set initrd_system="kiwi"
        ''')
        build_type_name = self.xml_state.get_build_type_name()
        boot_image_reference = self.xml_state.build_type.get_boot()
        if build_type_name == 'vmx' and boot_image_reference:
            initrd_system = self.xml_state.build_type.get_initrd_system()
            if initrd_system and initrd_system == 'dracut':
                raise KiwiRuntimeError(
                    message.format(
                        initrd_system, boot_image_reference, build_type_name
                    )
                )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# kiwi documentation build configuration file, created by
# sphinx-quickstart on Fri Feb  5 11:03:18 2016.
#
import sys
from os.path import abspath, dirname, join, normpath
import shlex
import sphinx_rtd_theme

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
_path = normpath(join(dirname(__file__), "../.."))
sys.path.insert(0, _path)

# autodoc imports all from kiwi, thus we need the global log
from kiwi import logger


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.extlinks',
    'sphinx.ext.todo',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinxcontrib.spelling',
    'sphinx.ext.autodoc'
]

docopt_ignore = [
    'kiwi.cli',
    'kiwi.tasks.system_build',
    'kiwi.tasks.system_prepare',
    'kiwi.tasks.system_update',
    'kiwi.tasks.system_create',
    'kiwi.tasks.result_list',
    'kiwi.tasks.result_bundle',
    'kiwi.tasks.image_resize',
    'kiwi.tasks.image_info'
]

def remove_module_docstring(app, what, name, obj, options, lines):
    if what == "module" and name in docopt_ignore:
        del lines[:]

def setup(app):
    app.connect("autodoc-process-docstring", remove_module_docstring)

spelling_lang = 'en_US'
spelling_show_suggestions = True
spelling_ignore_acronyms = True
spelling_ignore_importable_modules = True
spelling_ignore_python_builtins = True
spelling_ignore_pypi_package_names = True
spelling_word_list_filename = 'spelling_wordlist.txt'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['.templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The encoding of source files.
source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

default_role="py:obj"

# General information about the project.
project = u'kiwi'
copyright = u'2016, Marcus Schäfer'
author = u'Marcus Schäfer'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = u'9.6.0'
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

extlinks = {
    'issue': ('https://github.com/SUSE/kiwi/issues/%s', '#'),
    'pr': ('https://github.com/SUSE/kiwi/pull/%s', 'PR #'),
    'ghkiwi': ('https://github.com/SUSE/kiwi/blob/master/%s', '')
}


autosummary_generate = True

# -- Options for HTML output ----------------------------------------------

#html_short_title = '%s-%s' % (project, version)
#html_last_updated_fmt = '%b %d, %Y'
#html_split_index = True
html_logo = 'img/kiwi-logo.png'

html_sidebars = {
   '**': [
          'localtoc.html', 'relations.html',
          'about.html', 'searchbox.html',
         ]
}

html_theme = "sphinx_rtd_theme"

html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

html_theme_options = {
    'collapse_navigation': False,
    'display_version': False
}

# -- Options for manual page output ---------------------------------------

# The man page toctree documents.
kiwi_doc = 'manual/kiwi'
result_list_doc = 'manual/result_list'
result_bundle_doc = 'manual/result_bundle'
system_prepare_doc = 'manual/system_prepare'
system_update_doc = 'manual/system_update'
system_build_doc = 'manual/system_build'
system_create_doc = 'manual/system_create'
image_resize_doc = 'manual/image_resize'
image_info_doc = 'manual/image_info'

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        kiwi_doc,
        'kiwi', u'Creating Operating System Images',
        [author],
        2
    ),
    (
        result_list_doc,
        'kiwi::result::list',
        u'List build results',
        [author],
        2
    ),
    (
        result_bundle_doc,
        'kiwi::result::bundle',
        u'Bundle build results',
        [author],
        2
    ),
    (
        system_prepare_doc,
        'kiwi::system::prepare',
        u'Prepare image root system',
        [author],
        2
    ),
    (
        system_create_doc,
        'kiwi::system::create',
        u'Create image from prepared root system',
        [author],
        2
    ),
    (
        system_update_doc,
        'kiwi::system::update',
        u'Update/Upgrade image root system',
        [author],
        2
    ),
    (
        system_build_doc,
        'kiwi::system::build',
        u'Build image in combined prepare and create step',
        [author],
        2
    ),
    (
        image_resize_doc,
        'kiwi::image::resize',
        u'Resize disk images to new geometry',
        [author],
        2
    ),
    (
        image_info_doc,
        'kiwi::image::info',
        u'Provide detailed information about an image description',
        [author],
        2
    )
]

# If true, show URL addresses after external links.
#man_show_urls = False



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from kiwi.bootloader.config.grub2 import BootLoaderConfigGrub2
from kiwi.bootloader.config.isolinux import BootLoaderConfigIsoLinux
from kiwi.bootloader.config.zipl import BootLoaderConfigZipl

from kiwi.exceptions import (
    KiwiBootLoaderConfigSetupError
)


class BootLoaderConfig(object):
    """
    BootLoaderConfig factory

    Attributes

    * :attr:`name`
        bootloader name

    * :attr:`xml_state`
        Instance of XMLState

    * :attr:`root_dir`
        root directory path name

    * :attr:`custom_args`
        list of custom bootloader arguments
    """
    def __new__(self, name, xml_state, root_dir, custom_args=None):
        if name == 'grub2':
            return BootLoaderConfigGrub2(
                xml_state, root_dir, custom_args
            )
        elif name == 'grub2_s390x_emu':
            return BootLoaderConfigZipl(
                xml_state, root_dir, custom_args
            )
        elif name == 'isolinux':
            return BootLoaderConfigIsoLinux(
                xml_state, root_dir, custom_args
            )
        else:
            raise KiwiBootLoaderConfigSetupError(
                'Support for %s bootloader config not implemented' % name
            )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from collections import namedtuple
import pickle
import os

# project
from kiwi.logger import log

from kiwi.exceptions import (
    KiwiResultError
)

# must be global to allow pickle to find it
result_file_type = namedtuple(
    'result_file_type', ['filename', 'use_for_bundle', 'compress', 'shasum']
)


class Result(object):
    """
    Collect image building results

    Attributes

    * :attr:`result_files`
        List of result files

    * :attr:`class_version`
        Result class version

    * :attr:`xml_state`
        Instance of XMLState
    """
    def __init__(self, xml_state):
        self.result_files = {}

        # Instances of this class are stored as result reference.
        # In order to handle class format changes any instance
        # provides a version information
        self.class_version = 1

        self.xml_state = xml_state

    def add(
        self, key, filename, use_for_bundle=True, compress=False, shasum=True
    ):
        """
        Add result tuple to result_files list

        :param string key: name
        :param string filename: file path name
        :param boot use_for_bundle: use when bundling results true|false
        :param bool compress: compress when bundling true|false
        :param bool shasum: create shasum when bundling true|false
        """
        if key and filename:
            self.result_files[key] = result_file_type(
                filename=filename,
                use_for_bundle=use_for_bundle,
                compress=compress,
                shasum=shasum
            )

    def get_results(self):
        """
        Current list of result tuples
        """
        return self.result_files

    def print_results(self):
        """
        Print results human readable
        """
        if self.result_files:
            log.info('Result files:')
            for key, value in sorted(list(self.result_files.items())):
                log.info('--> %s: %s', key, value.filename)

    def dump(self, filename):
        """
        Picke dump this instance to a file

        :param string filename: file path name
        """
        try:
            with open(filename, 'wb') as result:
                pickle.dump(self, result)
        except Exception as e:
            raise KiwiResultError(
                'Failed to pickle dump results: %s' % format(e)
            )

    @classmethod
    def load(self, filename):
        """
        Load pickle dumped filename into a Result instance

        :param string filename: file path name
        """
        if not os.path.exists(filename):
            raise KiwiResultError(
                'No result information %s found' % filename
            )
        try:
            with open(filename, 'rb') as result:
                return pickle.load(result)
        except Exception as e:
            raise KiwiResultError(
                'Failed to pickle load results: %s' % type(e).__name__
            )
